@@ -26,16 +26,12 @@ posedge -- callable to model a rising edge on a signal in a yield statement
 negedge -- callable to model a falling edge on a signal in a yield statement
 
 """
-from __future__ import absolute_import
-from __future__ import print_function
-
-from inspect import currentframe, getouterframes
 from copy import copy, deepcopy
-import operator
 
-from myhdl._compat import integer_types, long
 from myhdl import _simulator as sim
-from myhdl._simulator import _signals, _siglist, _futureEvents, now
+from myhdl._simulator import _futureEvents
+from myhdl._simulator import _siglist
+from myhdl._simulator import _signals
 from myhdl._intbv import intbv
 from myhdl._bin import bin
 
@@ -43,7 +39,7 @@ from myhdl._bin import bin
 
 _schedule = _futureEvents.append
 
-   
+
 def _isListOfSigs(obj):
     """ Check if obj is a non-empty list of signals. """
     if isinstance(obj, list) and len(obj) > 0:
@@ -53,8 +49,8 @@ def _isListOfSigs(obj):
         return True
     else:
         return False
-     
-       
+
+
 class _WaiterList(list):
 
     def purge(self):
@@ -63,18 +59,25 @@ class _WaiterList(list):
 
 
 class _PosedgeWaiterList(_WaiterList):
+
     def __init__(self, sig):
         self.sig = sig
+
     def _toVerilog(self):
         return "posedge %s" % self.sig._name
+
     def _toVHDL(self):
         return "rising_edge(%s)" % self.sig._name
-    
+
+
 class _NegedgeWaiterList(_WaiterList):
+
     def __init__(self, sig):
         self.sig = sig
+
     def _toVerilog(self):
         return "negedge %s" % self.sig._name
+
     def _toVHDL(self):
         return "falling_edge(%s)" % self.sig._name
 
@@ -83,11 +86,14 @@ def posedge(sig):
     """ Return a posedge trigger object """
     return sig.posedge
 
+
 def negedge(sig):
     """ Return a negedge trigger object """
     return sig.negedge
 
 # signal factory function
+
+
 def Signal(val=None, delay=None):
     """ Return a new _Signal (default or delay 0) or DelayedSignal """
     if delay is not None:
@@ -96,7 +102,8 @@ def Signal(val=None, delay=None):
         return _DelayedSignal(val, delay)
     else:
         return _Signal(val)
-    
+
+
 class _Signal(object):
 
     """ _Signal class.
@@ -109,26 +116,25 @@ class _Signal(object):
 
     __slots__ = ('_next', '_val', '_min', '_max', '_type', '_init',
                  '_eventWaiters', '_posedgeWaiters', '_negedgeWaiters',
-                 '_code', '_tracing', '_nrbits', '_checkVal', 
-                 '_setNextVal', '_copyVal2Next', '_printVcd', 
-                 '_driven' ,'_read', '_name', '_used', '_inList',
+                 '_code', '_tracing', '_nrbits', '_checkVal',
+                 '_setNextVal', '_copyVal2Next', '_printVcd',
+                 '_driven', '_read', '_name', '_used', '_inList',
                  '_waiter', 'toVHDL', 'toVerilog', '_slicesigs',
                  '_numeric'
-                )
-
+                 )
 
     def __init__(self, val=None):
         """ Construct a signal.
 
         val -- initial value
-        
+
         """
         self._init = deepcopy(val)
         self._val = deepcopy(val)
         self._next = deepcopy(val)
         self._min = self._max = None
-        self._name = self._read = self._driven = None
-        self._used = False
+        self._name = self._driven = None
+        self._read = self._used = False
         self._inList = False
         self._nrbits = 0
         self._numeric = True
@@ -138,8 +144,8 @@ class _Signal(object):
             self._setNextVal = self._setNextBool
             self._printVcd = self._printVcdBit
             self._nrbits = 1
-        elif isinstance(val, integer_types):
-            self._type = integer_types
+        elif isinstance(val, int):
+            self._type = (int,)
             self._setNextVal = self._setNextInt
         elif isinstance(val, intbv):
             self._type = intbv
@@ -173,11 +179,13 @@ class _Signal(object):
         del self._negedgeWaiters[:]
         self._val = deepcopy(self._init)
         self._next = deepcopy(self._init)
-        self._name = self._read = self._driven = None
+        self._name = self._driven = None
+        self._read = False  # dont clear self._used
+        self._inList = False
         self._numeric = True
         for s in self._slicesigs:
             s._clear()
-        
+
     def _update(self):
         val, next = self._val, self._next
         if val != next:
@@ -193,7 +201,7 @@ class _Signal(object):
                 self._val = None
             elif isinstance(val, intbv):
                 self._val._val = next._val
-            elif isinstance(val, (integer_types, EnumItemType)):
+            elif isinstance(val, (int, EnumItemType)):
                 self._val = next
             else:
                 self._val = deepcopy(next)
@@ -211,8 +219,8 @@ class _Signal(object):
     # support for the 'next' attribute
     @property
     def next(self):
-#        if self._next is self._val:
-#            self._next = deepcopy(self._val)
+        #        if self._next is self._val:
+        #            self._next = deepcopy(self._val)
         _siglist.append(self)
         return self._next
 
@@ -227,12 +235,12 @@ class _Signal(object):
     @property
     def posedge(self):
         return self._posedgeWaiters
-                       
+
     # support for the 'negedge' attribute
     @property
     def negedge(self):
         return self._negedgeWaiters
-    
+
     # support for the 'min' and 'max' attribute
     @property
     def max(self):
@@ -249,10 +257,10 @@ class _Signal(object):
 
     @driven.setter
     def driven(self, val):
-        if not val  in ("reg", "wire", True):
+        if not val in ("reg", "wire", True):
             raise ValueError('Expected value "reg", "wire", or True, got "%s"' % val)
         self._driven = val
-    
+
     # support for the 'read' attribute
     @property
     def read(self):
@@ -260,7 +268,7 @@ class _Signal(object):
 
     @read.setter
     def read(self, val):
-        if not val in (True, ):
+        if not val in (True,):
             raise ValueError('Expected value True, got "%s"' % val)
         self._markRead()
 
@@ -282,14 +290,14 @@ class _Signal(object):
     def _setNextInt(self, val):
         if isinstance(val, intbv):
             val = val._val
-        elif not isinstance(val, (integer_types, intbv)):
+        elif not isinstance(val, (int, intbv)):
             raise TypeError("Expected int or intbv, got %s" % type(val))
         self._next = val
 
     def _setNextIntbv(self, val):
         if isinstance(val, intbv):
             val = val._val
-        elif not isinstance(val, integer_types):
+        elif not isinstance(val, int):
             raise TypeError("Expected int or intbv, got %s" % type(val))
         self._next._val = val
         self._next._handleBounds()
@@ -297,17 +305,17 @@ class _Signal(object):
     def _setNextNonmutable(self, val):
         if not isinstance(val, self._type):
             raise TypeError("Expected %s, got %s" % (self._type, type(val)))
-        self._next = val    
-        
+        self._next = val
+
     def _setNextMutable(self, val):
         if not isinstance(val, self._type):
             raise TypeError("Expected %s, got %s" % (self._type, type(val)))
-        self._next = deepcopy(val)         
+        self._next = deepcopy(val)
 
     # vcd print methods
     def _printVcdStr(self):
         print("s%s %s" % (str(self._val), self._code), file=sim._tf)
-        
+
     def _printVcdHex(self):
         if self._val is None:
             print("sz %s" % self._code, file=sim._tf)
@@ -322,7 +330,7 @@ class _Signal(object):
 
     def _printVcdVec(self):
         if self._val is None:
-            print("b%s %s" % ('z'*self._nrbits, self._code), file=sim._tf)
+            print("b%s %s" % ('z' * self._nrbits, self._code), file=sim._tf)
         else:
             print("b%s %s" % (bin(self._val, self._nrbits), self._code), file=sim._tf)
 
@@ -332,13 +340,11 @@ class _Signal(object):
         self._slicesigs.append(s)
         return s
 
-
     ### operators for which delegation to current value is appropriate ###
-        
+
     def __hash__(self):
         raise TypeError("Signals are unhashable")
-        
-    
+
     def __bool__(self):
         return bool(self._val)
 
@@ -353,7 +359,7 @@ class _Signal(object):
 
     def __getitem__(self, key):
         return self._val[key]
-        
+
     # integer-like methods
 
     def __add__(self, other):
@@ -361,14 +367,16 @@ class _Signal(object):
             return self._val + other._val
         else:
             return self._val + other
+
     def __radd__(self, other):
         return other + self._val
-    
+
     def __sub__(self, other):
         if isinstance(other, _Signal):
             return self._val - other._val
         else:
             return self._val - other
+
     def __rsub__(self, other):
         return other - self._val
 
@@ -377,6 +385,7 @@ class _Signal(object):
             return self._val * other._val
         else:
             return self._val * other
+
     def __rmul__(self, other):
         return other * self._val
 
@@ -385,32 +394,36 @@ class _Signal(object):
             return self._val / other._val
         else:
             return self._val / other
+
     def __rtruediv__(self, other):
         return other / self._val
-    
+
     def __floordiv__(self, other):
         if isinstance(other, _Signal):
             return self._val // other._val
         else:
             return self._val // other
+
     def __rfloordiv__(self, other):
-        return other //  self._val
-    
+        return other // self._val
+
     def __mod__(self, other):
         if isinstance(other, _Signal):
             return self._val % other._val
         else:
             return self._val % other
+
     def __rmod__(self, other):
         return other % self._val
 
     # XXX divmod
-    
+
     def __pow__(self, other):
         if isinstance(other, _Signal):
             return self._val ** other._val
         else:
             return self._val ** other
+
     def __rpow__(self, other):
         return other ** self._val
 
@@ -419,22 +432,25 @@ class _Signal(object):
             return self._val << other._val
         else:
             return self._val << other
+
     def __rlshift__(self, other):
         return other << self._val
-            
+
     def __rshift__(self, other):
         if isinstance(other, _Signal):
             return self._val >> other._val
         else:
             return self._val >> other
+
     def __rrshift__(self, other):
         return other >> self._val
-           
+
     def __and__(self, other):
         if isinstance(other, _Signal):
             return self._val & other._val
         else:
             return self._val & other
+
     def __rand__(self, other):
         return other & self._val
 
@@ -443,17 +459,19 @@ class _Signal(object):
             return self._val | other._val
         else:
             return self._val | other
+
     def __ror__(self, other):
         return other | self._val
-    
+
     def __xor__(self, other):
         if isinstance(other, _Signal):
             return self._val ^ other._val
         else:
             return self._val ^ other
+
     def __rxor__(self, other):
         return other ^ self._val
-    
+
     def __neg__(self):
         return -self._val
 
@@ -465,48 +483,51 @@ class _Signal(object):
 
     def __invert__(self):
         return ~self._val
-        
+
     # conversions
-    
+
     def __int__(self):
         return int(self._val)
-        
+
     def __long__(self):
-        return long(self._val)
+        return int(self._val)
 
     def __float__(self):
         return float(self._val)
-    
+
     def __oct__(self):
         return oct(self._val)
-    
+
     def __hex__(self):
         return hex(self._val)
-    
+
     def __index__(self):
         return int(self._val)
 
-
     # comparisons
     def __eq__(self, other):
-        return self.val == other 
+        return self.val == other
+
     def __ne__(self, other):
-        return self.val != other 
+        return self.val != other
+
     def __lt__(self, other):
         return self.val < other
+
     def __le__(self, other):
         return self.val <= other
+
     def __gt__(self, other):
         return self.val > other
+
     def __ge__(self, other):
         return self.val >= other
 
-
     # method lookup delegation
     def __getattr__(self, attr):
         return getattr(self._val, attr)
 
-    # representation 
+    # representation
     def __str__(self):
         if self._name:
             return self._name
@@ -527,13 +548,11 @@ class _Signal(object):
     __ior__ = __iand__ = __ixor__ = __irshift__ = __ilshift__ = _augm
     __itruediv__ = __ifloordiv__ = _augm
 
-
     # index and slice assignment not supported
     def __setitem__(self, key, val):
         raise TypeError("Signal object doesn't support item/slice assignment")
 
-
-    # continues assignment support
+    # continuous assignment support
     def assign(self, sig):
 
         self.driven = "wire"
@@ -556,9 +575,9 @@ class _Signal(object):
 
 
 class _DelayedSignal(_Signal):
-    
+
     __slots__ = ('_nextZ', '_delay', '_timeStamp',
-                )
+                 )
 
     def __init__(self, val=None, delay=1):
         """ Construct a new DelayedSignal.
@@ -594,7 +613,7 @@ class _DelayedSignal(_Signal):
             self._val = copy(next)
             if self._tracing:
                 self._printVcd()
-            return waiters            
+            return waiters
         else:
             return []
 
@@ -607,15 +626,49 @@ class _DelayedSignal(_Signal):
     def delay(self, delay):
         self._delay = delay
 
-        
+
 class _SignalWrap(object):
+
     def __init__(self, sig, next, timeStamp):
         self.sig = sig
         self.next = next
         self.timeStamp = timeStamp
+
     def apply(self):
         return self.sig._apply(self.next, self.timeStamp)
 
+
+class Constant(_Signal):
+    ''' effective constants '''
+
+    def __init__(self, val=None):
+        super(Constant, self).__init__(val)
+
+    # override some essentials
+    def __repr__(self):
+        return "Constant(" + repr(self._val) + ")"
+
+    # there is support for the 'next' attribute
+    @property
+    def next(self):
+        return None
+
+    @next.setter
+    def next(self, val):
+        raise PermissionError("A 'Constant' can not be changed!")
+
+    # neither can it be driven
+    # support for the 'driven' attribute
+    @property
+    def driven(self):
+        return None
+
+    @driven.setter
+    def driven(self, val):
+        # quietly ignore?
+        pass
+
+
 # for export
 SignalType = _Signal
 
