@@ -1,6 +1,6 @@
-from _verify import verify, analyze, registerSimulator
-from _toVerilog import toVerilog
-from _toVHDL import toVHDL
+from ._verify import verify, analyze, registerSimulator
+from ._toVerilog import toVerilog
+from ._toVHDL import toVHDL
 
 __all__ = ["verify",
            "analyze",
