@@ -1,3 +1,4 @@
+import myhdl
 from myhdl import *
 from myhdl.conversion import analyze
 
@@ -33,6 +34,7 @@ def simulate(timesteps):
     tb = traceSignals(test_dff)
     sim = Simulation(tb)
     sim.run(timesteps)
+    sim.quit()
 
 simulate(2000)
 
