@@ -1,3 +1,4 @@
+import myhdl
 from myhdl import *
 
 def dffa(q, d, clk, rst):
@@ -44,6 +45,7 @@ def simulate(timesteps):
     tb = traceSignals(test_dffa)
     sim = Simulation(tb)
     sim.run(timesteps)
+    sim.quit()
 
 simulate(20000)
 
